/* packet-rawss7.c
 * Routines for Raw SS7 layer of Ethereal dissection
 * Copyright 2003, Navin Anand <navinanand[AT]winphoria.com>
 * Guidance by Martin Regner is acknowledged, thank you Martin Regner.
 * Some updates by Jeff Morriss <jeff.morriss[AT]ulticom.com>
 *
 * For more information, see "doc/README.rawss7"
 *
 * $Id$
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>

#include <rawss7.h>

const value_string rawss7_vals[] = {
	{LINKTYPE_RAWSS7_MTP2,	"MTP2"	},
	{LINKTYPE_RAWSS7_MTP3,	"MTP3"	},
	{LINKTYPE_RAWSS7_SCCP,	"SCCP"	},
	{0,			NULL	} };

/* Initialize the protocol and registered fields */
static int proto_rawss7 = -1;
static int hf_rawss7_type = -1;
static int hf_rawss7_length = -1;
static int hf_rawss7_spare = -1;

/* Initialize the subtree pointers */
static gint ett_rawss7 = -1;

static dissector_handle_t data_handle;

static dissector_table_t rawss7_type_dissector_table;

/* Code to actually dissect the packets */
static void
dissect_rawss7(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	rawss7_hdr rawss7hdr;
	proto_item *ti = NULL;
	proto_tree *rawss7_tree = NULL;
	gint data_length;
	tvbuff_t *data_tvb;

	/* Make entries in Protocol column and Info column on summary display */
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "Raw SS7");

	if (check_col(pinfo->cinfo, COL_INFO))
		col_set_str(pinfo->cinfo, COL_INFO, "Data");

	rawss7hdr.type = tvb_get_ntohl(tvb, RAWSS7_TYPE_OFFSET);

	if (tree) {

		/* create display subtree for the protocol */
		ti = proto_tree_add_item(tree, proto_rawss7, tvb, 0, -1, FALSE);

		rawss7_tree = proto_item_add_subtree(ti, ett_rawss7);

		proto_tree_add_item(rawss7_tree, hf_rawss7_type, tvb,
				    RAWSS7_TYPE_OFFSET,
				    RAWSS7_TYPE_LENGTH, FALSE);

		proto_tree_add_item(rawss7_tree, hf_rawss7_length, tvb,
				    RAWSS7_LENGTH_OFFSET,
				    RAWSS7_LENGTH_LENGTH, FALSE);

		proto_tree_add_item(rawss7_tree, hf_rawss7_spare, tvb,
				    RAWSS7_SPARE_OFFSET,
				    RAWSS7_SPARE_LENGTH, FALSE);
	}

	data_length = tvb_length(tvb) - RAWSS7_HEADER_SIZE;
	data_tvb = tvb_new_subset(tvb, RAWSS7_DATA_OFFSET, data_length,
				  data_length);

	if (dissector_try_port(rawss7_type_dissector_table, rawss7hdr.type,
			       data_tvb, pinfo, tree))
		return;

	call_dissector(data_handle, data_tvb, pinfo, tree);
}


/* Register the protocol with Ethereal */
void
proto_register_rawss7(void)
{

	/* Setup list of header fields */
	static hf_register_info hf[] = {
		{ &hf_rawss7_type,
			{ "Protocol type", "rawss7.type",
			FT_UINT32, BASE_DEC, VALS(rawss7_vals), 0x0,
			"Protocol type", HFILL }
		},
		{ &hf_rawss7_length,
			{ "Packet length", "rawss7.length",
			FT_UINT16, BASE_DEC, NULL, 0x0,
			"Packet length", HFILL }
		},
		{ &hf_rawss7_spare,
			{ "Spare", "rawss7.spare",
			FT_UINT16, BASE_DEC, NULL, 0x0,
			"Spare", HFILL }
		}
	};

	/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_rawss7,
	};

	/* Register the protocol name and description */
	proto_rawss7 = proto_register_protocol("Raw SS7 layer",
					       "RawSS7", "rawss7");

	register_dissector("rawss7", dissect_rawss7, proto_rawss7);

	proto_register_field_array(proto_rawss7, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));

	/* subdissector code */
	rawss7_type_dissector_table =
		register_dissector_table("rawss7.type", "Protocol type",
					 FT_UINT16, BASE_HEX);
}


void
proto_reg_handoff_rawss7(void)
{
	dissector_handle_t rawss7_handle;

	data_handle = find_dissector("data");
	rawss7_handle = create_dissector_handle(dissect_rawss7,
						  proto_rawss7);

	dissector_add("wtap_encap", WTAP_ENCAP_RAWSS7, rawss7_handle);

}

